using System;
using System.Text;
using System.IO;
using System.Net;
using System.Collections.Generic;
using Terrasoft.Nui.ServiceModel.DataContract;
using Terrasoft.Core.Entities;
using System.Web.Script.Serialization;


namespace DataServiceUpdateExample
{
    class Program
    {
        // Основной Uri приложения bpm'online. Необходимо заменить на пользовательский.
        private const string baseUri = @"http://example.bpmonline.com";
        // Строка запроса к методу Login сервиса AuthService.svc.
        private const string authServiceUri = baseUri + @"/ServiceModel/AuthService.svc/Login";
        // Строка пути запроса DeleteQuery.
        private const string deleteQueryUri = baseUri + @"/0/DataService/json/reply/DeleteQuery";
        // Cookie аутентификации bpm'online.
        private static CookieContainer AuthCookie = new CookieContainer();

        // Метод выполняет аутентификацию пользователя.
        // Параметры:
        // userName - имя пользователя bpm'online,
        // userPassword - пароль пользователя bpm'online.
        private static bool TryLogin(string userName, string userPassword)
        {
            // Создание экземпляра запроса к сервису аутентификации.
            var authRequest = HttpWebRequest.Create(authServiceUri) as HttpWebRequest;
            // Определение метода HTTP-запроса.
            authRequest.Method = "POST";
            // Определение типа контента запроса.
            authRequest.ContentType = "application/json";
            // Включение использования cookie в запросе.
            authRequest.CookieContainer = AuthCookie;
            // Помещение в тело запроса учетной информации пользователя.
            using (var requesrStream = authRequest.GetRequestStream())
            {
                using (var writer = new StreamWriter(requesrStream))
                {
                    writer.Write(@"{
                        ""UserName"":""" + userName + @""",
                        ""UserPassword"":""" + userPassword + @"""
                    }");
                }
            }

            // Получение ответа от сервера. Если аутентификация проходит успешно, в свойство AuthCookie будут 
            // помещены cookie, которые могут быть использованы для последующих запросов.
            using (var response = (HttpWebResponse)authRequest.GetResponse())
            {
                if (AuthCookie.Count > 0)
                {
                    return true;
                }
            }
            return false;
        }

        // Главный метод приложения.
        static void Main(string[] args)
        {
            //Если выполнение аутентификации неудачно, приложение завершает работу.
            if (!TryLogin("Пользователь 1", "Пользователь 1"))
            {
                Console.WriteLine("Ошибка аутентификации!");
                return;
            }
            // Экземпляр класса запроса.
            var deleteQuery = new DeleteQuery()
            {
                // Название корневой схемы.
                RootSchemaName = "Contact",
                // Фильтры запроса.
                Filters = new Filters()
                {
                    // Тип фильтра — группа.
                    FilterType = Terrasoft.Nui.ServiceModel.DataContract.FilterType.FilterGroup,
                    // Коллекция фильтров.
                    Items = new Dictionary<string, Filter>()
                    {
                         // Фильтрация по имени.
                        {
                            // Ключ.
                            "FilterByName",
                            // Значение.
                            new Filter
                            {
                                // Тип фильтра — фильтр сравнения.
                                FilterType = Terrasoft.Nui.ServiceModel.DataContract.FilterType.CompareFilter,
                                // Тип сравнения — начинается выражением.
                                ComparisonType = FilterComparisonType.Equal,
                                // Выражение, подлежащее проверке.
                                LeftExpression = new BaseExpression()
                                {
                                    // Тип выражения — колонка схемы.
                                    ExpressionType = EntitySchemaQueryExpressionType.SchemaColumn,
                                    // Пкть к колонке.
                                    ColumnPath = "Name"
                                },
                                // Выражение фильтрации.
                                RightExpression = new BaseExpression()
                                {
                                    // Тип выражения — параметр.
                                    ExpressionType = EntitySchemaQueryExpressionType.Parameter,
                                     // Параметр выражения.
                                    Parameter = new Parameter()
                                    {
                                        // Тип данных параметра — текст.
                                        DataValueType = DataValueType.Text,
                                        // Значение параметра.
                                        Value = "Иванов Иван Иванович"
                                    }
                                }
                            }
                        }
                    }
                }
            };

            // Сериализация экземпляра класса запроса на чтение в JSON-строку.
            var json = new JavaScriptSerializer().Serialize(deleteQuery);

            Console.WriteLine(json);
            Console.WriteLine();
            Console.ReadLine();

            // Преобразование строки JSON-объекта в массив байтов.
            byte[] jsonArray = Encoding.UTF8.GetBytes(json);
            // Создание экземпляра HTTP-запроса.
            var deleteRequest = HttpWebRequest.Create(deleteQueryUri) as HttpWebRequest;
            // Определение метода запроса.
            deleteRequest.Method = "POST";
            // Определение типа содержимого запроса.
            deleteRequest.ContentType = "application/json";
            // Добавление полученных ранее аутентификационных cookie в запрос на получение данных.
            deleteRequest.CookieContainer = AuthCookie;
            // Set the ContentLength property of the WebRequest.
            deleteRequest.ContentLength = jsonArray.Length;

            // Помещение в содержимое запроса JSON-объекта.
            using (var requestStream = deleteRequest.GetRequestStream())
            {
                requestStream.Write(jsonArray, 0, jsonArray.Length);
            }
            // Выполнение HTTP запроса и получение ответа от сервера.
            using (var response = (HttpWebResponse)deleteRequest.GetResponse())
            {
                // Вывод ответа в консоль.
                using (StreamReader reader = new StreamReader(response.GetResponseStream()))
                {
                    Console.WriteLine(reader.ReadToEnd());
                }
            }
            // Задержка выполнения приложения.
            Console.ReadKey();

        }
    }
}
